# CELERY
# Default broker URL. See http://celery.readthedocs.org/en/latest/configuration.html#broker-url.
BROKER_URL = 'amqp://celery:celery@127.0.0.1:5672'

# Disable connection pooling. Connections may be severed by load balancers.
# This forces the application to connect explicitly to the broker each time
# rather than assume a long-lived connection.
BROKER_POOL_LIMIT = 0
BROKER_CONNECTION_TIMEOUT = 1

# Use heartbeats to prevent broker connection loss. When the broker
# is behind a load balancer, the load balancer may timeout Celery's
# connection to the broker, causing messages to be lost.
BROKER_HEARTBEAT = 10.0
BROKER_HEARTBEAT_CHECKRATE = 2

# Backend used to store task results.
# See http://celery.readthedocs.org/en/latest/configuration.html#celery-result-backend.
CELERY_RESULT_BACKEND = None

# A sequence of modules to import when the worker starts.
# See http://celery.readthedocs.org/en/latest/configuration.html#celery-imports.
CELERY_IMPORTS = (
    'ecommerce_worker.fulfillment.v1.tasks',
    'ecommerce_worker.sailthru.v1.tasks',
)

DEFAULT_PRIORITY_QUEUE = 'ecommerce.default'
CELERY_DEFAULT_EXCHANGE = 'ecommerce'
CELERY_DEFAULT_ROUTING_KEY = 'ecommerce'
CELERY_DEFAULT_QUEUE = DEFAULT_PRIORITY_QUEUE
# Prevent Celery from removing handlers on the root logger. Allows setting custom logging handlers.
# See http://celery.readthedocs.org/en/latest/configuration.html#celeryd-hijack-root-logger.
CELERYD_HIJACK_ROOT_LOGGER = False

# Specify allowed serializers that are consistent with Celery 3 defaults
CELERY_TASK_SERIALIZER = 'pickle'
CELERY_RESULT_SERIALIZER = 'pickle'
CELERY_EVENT_SERIALIZER = 'json'
CELERY_ACCEPT_CONTENT = ['json', 'pickle', 'yaml']
# END CELERY


# ORDER FULFILLMENT
# Absolute URL used to construct API calls against the ecommerce service.
ECOMMERCE_API_ROOT = 'http://127.0.0.1:8002/api/v2/'

# Maximum number of retries before giving up on the fulfillment of an order.
# For reference, 11 retries with exponential backoff yields a maximum waiting
# time of 2047 seconds (about 30 minutes). Defaulting this to None could yield
# unwanted behavior: infinite retries.
MAX_FULFILLMENT_RETRIES = 11
# END ORDER FULFILLMENT

# AUTHENTICATION
JWT_SECRET_KEY = 'insecure-secret-key'
JWT_ISSUER = 'ecommerce_worker'

ECOMMERCE_SERVICE_USERNAME = 'ecommerce_worker'
# END AUTHENTICATION

# Site Overrides provide support for site/partner-specific configuration settings where applicable
# For example, the ECOMMERCE_API_ROOT value is different from one ecommerce site to the next
SITE_OVERRIDES = None

# Settings for Sailthru email marketing integration
SAILTHRU = {
    # Set to false to ignore Sailthru events
    'SAILTHRU_ENABLE': False,

    # Template used when user upgrades to verified
    'SAILTHRU_UPGRADE_TEMPLATE': None,

    # Template used with user purchases a course
    'SAILTHRU_PURCHASE_TEMPLATE': None,

    # Template used with user enrolls in a free course
    'SAILTHRU_ENROLL_TEMPLATE': None,

    # Abandoned cart template
    'SAILTHRU_ABANDONED_CART_TEMPLATE': None,

    # minutes to delay before abandoned cart message
    'SAILTHRU_ABANDONED_CART_DELAY': 60,

    # Sailthru key and secret required for integration
    'SAILTHRU_KEY': 'sailthru key here',
    'SAILTHRU_SECRET': 'sailthru secret here',

    # Retry settings for Sailthru celery tasks
    'SAILTHRU_RETRY_SECONDS': 3600,
    'SAILTHRU_RETRY_ATTEMPTS': 6,

    # ttl for cached course content from Sailthru (in seconds)
    'SAILTHRU_CACHE_TTL_SECONDS': 3600,

    # dummy price for audit/honor (i.e., if cost = 0)
    # Note: setting this value to 0 skips Sailthru calls for free transactions
    'SAILTHRU_MINIMUM_COST': 100,

    # Transactional email template name map
    'templates': {
        'course_refund': 'Course Refund',
        'assignment_email': 'Offer Assignment Email',
        'enterprise_portal_email': 'Enterprise Portal Email',
    }
}



"""
Notification class for sending the notification emails.
"""
from __future__ import absolute_import
from celery.utils.log import get_task_logger

from sailthru.sailthru_error import SailthruClientError
from ecommerce_worker.sailthru.v1.exceptions import SailthruError
from ecommerce_worker.sailthru.v1.utils import can_retry_sailthru_request, get_sailthru_client

log = get_task_logger(__name__)


class Notification(object):
    """
    This class exports the 'send' function for sending the emails.
    """

    def __init__(self, config, emails, email_vars, logger_prefix, site_code, template):
        self.config = config
        self.emails = emails
        self.email_vars = email_vars
        self.logger_prefix = logger_prefix
        self.site_code = site_code
        self.template = template

    def _is_eligible_for_retry(self, response):
        """
        Return a bool whether this task is eligible for retry or not.
        Also log the appropriate message according to occurred error.
        """
        is_eligible_for_retry = False
        error = response.get_error()
        log.error(
            '[{logger_prefix}] An error occurred while attempting to send a notification. Message: {message},'
            ' Error code: {token_error_code}, Error Message: {token_error_message}.'.format(
                logger_prefix=self.logger_prefix,
                message=self.email_vars.get('email_body'),
                token_error_code=error.get_error_code(),
                token_error_message=error.get_message()
            )
        )
        if can_retry_sailthru_request(error):
            log.info(
                '[{logger_prefix}] An attempt will be made to resend the notification.'
                ' Message: {message}'.format(
                    logger_prefix=self.logger_prefix,
                    message=self.email_vars.get('email_body')
                )
            )
            is_eligible_for_retry = True
        else:
            log.warning(
                '[{logger_prefix}] No further attempts will be made to send the notification.'
                ' Failed Message: {message}'.format(
                    logger_prefix=self.logger_prefix,
                    message=self.email_vars.get('email_body')
                )
            )
        return is_eligible_for_retry

    @staticmethod
    def _get_send_callback(is_multi_send):
        """
        Return the associated function 'send' in case of single send and 'multi_send' in case of multi_send
        """
        return 'multi_send' if is_multi_send else 'send'

    def _get_client(self):
        """
        Return the sailthru client or None if exception raised.
        """
        sailthru_client = None
        try:
            sailthru_client = get_sailthru_client(self.site_code)
        except SailthruError:
            log.exception(
                '[{logger_prefix}] A client error occurred while attempting to send a notification.'
                ' Message: {message}'.format(
                    logger_prefix=self.logger_prefix,
                    message=self.email_vars.get('email_body')
                )
            )
        return sailthru_client

    def _get_params(self, is_multi_send):
        """
        Return the dict of parameters according to the given 'is_multi_send' parameter.

        It can be a simple 'send' function in which we will send an email
        to a single email address or it can be a 'multi_send' function.
        """
        params = {
            'template': self.config['templates'][self.template],
            '_vars': self.email_vars
        }
        email_param = 'emails' if is_multi_send else 'email'
        params[email_param] = self.emails
        return params

    def send(self, is_multi_send=False):
        """
        Send the notification email to single email address or comma
        separated emails on bases of is_multi_send parameter

        Returns:
            response: Sailthru endpoint response.
            is_eligible_for_retry(Bool): whether this response is eligible for retry or not.
        """
        is_eligible_for_retry = False
        sailthru_client = self._get_client()
        if sailthru_client is None:
            return None, is_eligible_for_retry

        try:
            response = getattr(
                sailthru_client,
                self._get_send_callback(is_multi_send)
            )(**self._get_params(is_multi_send))
        except SailthruClientError:
            log.exception(
                '[{logger_prefix}] A client error occurred while attempting to send a notification.'
                ' Message: {message}'.format(
                    logger_prefix=self.logger_prefix,
                    message=self.email_vars.get('email_body')
                )
            )
            return None, is_eligible_for_retry

        if response.is_ok():
            log.info(
                '[{logger_prefix}] A notification has been sent successfully. Message: {message}'.format(
                    logger_prefix=self.logger_prefix,
                    message=self.email_vars.get('email_body')
                )
            )
        else:
            is_eligible_for_retry = self._is_eligible_for_retry(response)
        return response, is_eligible_for_retry



from __future__ import absolute_import
from setuptools import setup, find_packages


with open('README.rst') as readme:
    long_description = readme.read()


def load_requirements(*requirements_paths):
    """
    Load all requirements from the specified requirements files.
    Returns a list of requirement strings.
    """
    requirements = set()
    for path in requirements_paths:
        requirements.update(
            line.split('#')[0].strip() for line in open(path).readlines()
            if is_requirement(line.strip())
        )
    return list(requirements)


def is_requirement(line):
    """
    Return True if the requirement line is a package requirement;
    that is, it is not blank, a comment, a URL, or an included file.
    """
    return not (
        line == '' or
        line.startswith('-c') or
        line.startswith('-r') or
        line.startswith('#') or
        line.startswith('-e') or
        line.startswith('git+')
    )


setup(
    name='edx-ecommerce-worker',
    version='0.10.0',
    description='Celery tasks supporting the operations of edX\'s ecommerce service',
    long_description=long_description,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet',
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
    ],
    keywords='edx ecommerce worker',
    url='https://github.com/edx/ecommerce-worker',
    author='edX',
    author_email='oscm@edx.org',
    license='AGPL',
    packages=find_packages(exclude=['*.tests']),
    install_requires=load_requirements('requirements/base.in'),
)
